#ifndef DATE_H
#define DATE_H

/*
 ******************************
 * Object Oriented Programming in C
 *
 * Author: Laurent Deniau, Laurent.Deniau@cern.ch
 *
 * For more information, please see the paper:
 * http://home.cern.ch/ldeniau/html/oopc/oopc.html
 *
 ******************************
 */

#include <time.h>
#include <ooc.h>

/*
  -------------------------
  date interface

  Exceptions thrown:
  ooc_bad_alloc (setDefaultFormat)

  Default date format is:
  "%d/%m/%Y %H:%M:%S"
  -------------------------
*/

#undef  OBJECT
#define OBJECT date

BASEOBJECT_INTERFACE

  time_t private(time);

BASEOBJECT_METHODS

  int constMethod(isValid);

  int  method_(set) int day,  int month, int year,
                    int hour, int min,   int sec __;
  void method_(setToTime) time_t const* a_date __;
  int  method_(setToDate) struct tm const* a_date __;
  int  method_(setToString) char const* a_date, char const* format __;
  int  method (setToNow);

  time_t constMethod(getTime);
  /* returns a pointer to a static date (struct tm) */
  struct tm const* constMethod(getDate);
  /* returns a pointer to a static string of maximum 64 character */
  char const* constMethod_(getString) char const* format __;

ENDOF_INTERFACE

CLASS_INTERFACE

  t_date *const classMethod_(new) int day,  int month, int year,
                                  int hour, int min,   int sec __;
  t_date *const classMethod_(newFromTime) time_t const* a_date __;
  t_date *const classMethod_(newFromDate) struct tm const* a_date __;
  t_date *const classMethod_(newFromString) char const* a_date, char const* format __;
  t_date *const classMethod (newFromNow);

  void   classMethod_(setDefaultFormat) char const* format __;
  void   classMethod_(copy) t_date *date1, t_date const* date2 __;
  /* returns difference in second */
  double classMethod_(diff) t_date const* date1, t_date const* date2 __;

ENDOF_INTERFACE

#endif /* DATE_H */
