/*
 ******************************
 * Object Oriented Programming in C
 *
 * Author: Laurent Deniau, Laurent.Deniau@cern.ch
 *
 * For more information, please see the paper:
 * http://home.cern.ch/ldeniau/html/oopc/oopc.html
 *
 ******************************
 */

#include <stdio.h>
#include <typeinfo>

/*
  Example of inheritance in C++:
  K --> J --> F
          --> G --> C --> A
                      --> B
          --> H --> D
                --> E --> B
          --> I
*/

#include <example.H>

int main(void)
{
  K *k = new K;

  printf("\nSize of objects:\n");        /* Output on 32-bits machine: */
  printf("sizeof(A) = %d\n", sizeof(A)); /* sizeof(A) = 4              */
  printf("sizeof(B) = %d\n", sizeof(B)); /* sizeof(B) = 4              */
  printf("sizeof(C) = %d\n", sizeof(C)); /* sizeof(C) = 8              */
  printf("sizeof(D) = %d\n", sizeof(D)); /* sizeof(D) = 4              */
  printf("sizeof(E) = %d\n", sizeof(E)); /* sizeof(E) = 4              */
  printf("sizeof(F) = %d\n", sizeof(F)); /* sizeof(F) = 4              */
  printf("sizeof(G) = %d\n", sizeof(G)); /* sizeof(G) = 8              */
  printf("sizeof(H) = %d\n", sizeof(H)); /* sizeof(H) = 8              */
  printf("sizeof(I) = %d\n", sizeof(I)); /* sizeof(I) = 4              */
  printf("sizeof(J) = %d\n", sizeof(J)); /* sizeof(J) = 24             */
  printf("sizeof(K) = %d\n", sizeof(K)); /* sizeof(K) = 24             */

  /* returns dynamic_cast<E>(static_cast<A>(k)) */
  printf("\nDynamic cast example1:\n\t%s @ %p (offset = %+d)\n",
	 typeid(dynamic_cast<E*>(static_cast<A*>(k))).name(),
	 dynamic_cast<E*>(static_cast<A*>(k)),
	 (size_t)((char*)dynamic_cast<E*>(static_cast<A*>(k)) - (char*)k));

  /* returns dynamic_cast<A>(static_cast<E>(k)) */
  printf("\nDynamic cast example2:\n\t%s @ %p (offset = %+d)\n",
	 typeid(dynamic_cast<A*>(static_cast<E*>(k))).name(),
	 dynamic_cast<A*>(static_cast<E*>(k)),
	 (size_t)((char*)dynamic_cast<A*>(static_cast<E*>(k)) - (char*)k));

  /* returns dynamic_cast<B>(static_cast<A>(k)) == NULL
     B appears twice in the class hierarchy
   */
  printf("\nDynamic cast example3:\n\t%s @ %p\n",
	 typeid(dynamic_cast<B*>(static_cast<A*>(k))).name(),
	 dynamic_cast<B*>(static_cast<A*>(k)));

  delete k;

  return 0;
}
